﻿using System;
using System.Diagnostics.CodeAnalysis;
using System.Threading.Tasks;

using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging;
using Rchlo.Omni.Common.Interfaces;
using Rchlo.Omni.Common.Models;
using Rchlo.Omni.Common.NetCoreApi.Attributes;
using Rchlo.Omni.Common.NetCoreApi.Extensions;

using RestApiTemplate.Application.Dtos.Request;
using RestApiTemplate.Application.Dtos.Response;
using RestApiTemplate.Application.Services.Interfaces;

namespace RestApiTemplate.Api.Controllers.v1
{
    /// <summary>
    /// Os controllers devem ser utilizados apenas como um fachada para as camadas de serviço onde devem ser implementadas
    /// as reras de negócio.
    /// SuppressSensitiveDataLog - Utilize esta notation para inibir os dados nos logs.
    /// ExcludeFromCodeCoverage -  Utilize esta notation para excluir a classe do CodeCoverage, 
    ///                             no caso a camada de Controller foi excluída do CodeCoverage pois não deve conter nenhum tipo de lógica 
    /// ToActionResult() - Extension de ActionResult utilizada por converter o ReturnMessage em ActionResult.
    /// </summary>
    [ApiVersion("1.0")]
    [Produces("application/json")]
    [Route("api/v{version:apiVersion}/[controller]")]
    [ExcludeFromCodeCoverage]
    public class ExampleController : ControllerBase
    {
        private readonly IExampleService _exampleService;
        private readonly IExampleDapperService _exampleDapperService;

        public ExampleController(IExampleService exampleService, IExampleDapperService exampleDapperService)
        {
            _exampleService = exampleService;
            _exampleDapperService = exampleDapperService;
        }

        /// <summary>
        /// Importante : Aqui deverá conter descrição EndPoint
        /// </summary>
        /// <param name="example">Descrição do parametro de entrada</param>
        /// <remarks>Alguma descrição mais detalhada sobre o serviço, caso necessário</remarks>
        /// <returns>Descrição do Retonro</returns>
        /// <response code="200">EndPoint executado com sucesso</response>
        /// <response code="500">Erro na execução do EndPoint</response> 
        [HttpGet]
        public async Task<ActionResult<ReturnMessage<IPagedList<ExampleResponse>>>> GetEnabledExamples([FromQuery]ExampleGetEnabledRequest example)
        {
            var returnMessage = await _exampleService.GetEnabledExamplesAsync(example).ConfigureAwait(true);
            return returnMessage.ToActionResult();
        }

        /// <summary>
        /// Importante : Aqui deverá conter descrição EndPoint
        /// </summary>
        /// <param name="example">Descrição do parametro de entrada</param>
        /// <remarks>
        ///     IMPORTANTE : 
        ///         Por definição interna, a diferença entre GetEnabledExamples e GetEnabledExamplesDapperAsync é a ORM utilizada em cada um dos casos : 
        ///         Dapper - GetEnabledExamplesDapperAsync -> Devemos utilizar o ORM Dapper apenas para Querys de Consultas mais complexas e robustas
        ///         EF - GetEnabledExamples -> Na maioria dos cenários utilizaremos o EF para CRUDs, assim ganhamos em tempo de desenvolvimento
        /// </remarks>
        /// <returns>Descrição do Retonro</returns>
        /// <response code="200">EndPoint executado com sucesso</response>
        /// <response code="500">Erro na execução do EndPoint</response> 
        [HttpGet("dapper")]
        public async Task<ActionResult<ReturnMessage<IPagedListCustom<ExampleResponse>>>> GetEnabledExamplesDapperAsync([FromQuery]ExampleGetEnabledRequest example)
        {
            var returnMessage = await _exampleDapperService.GetEnabledExamplesAsync(example).ConfigureAwait(false);
            return returnMessage.ToActionResult();
        }

        [HttpPost]
        [SuppressSensitiveDataLog]
        public async Task<ActionResult<ReturnMessage<ExampleResponse>>> Post(ExampleRequest example)
        {
            var returnMessage = await _exampleService.InsertExampleAsync(example).ConfigureAwait(false);
            return returnMessage.ToActionResult();
        }

        [HttpPut]
        public async Task<ActionResult<ReturnMessage<ExampleResponse>>> Put(Guid exampleId)
        {
            var returnMessage = await _exampleService.EnableExampleAsync(exampleId).ConfigureAwait(false);
            return returnMessage.ToActionResult();
        }

       
    }
}