﻿using System;
using System.Diagnostics.CodeAnalysis;
using Destructurama;

using FluentValidation.AspNetCore;

using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;

using Rchlo.Omni.Common.Extensions;
using Rchlo.Omni.Common.NetCoreApi.Extensions;

using RestApiTemplate.CrossCutting.AppSettings;
using RestApiTemplate.CrossCutting.Automapper;
using RestApiTemplate.CrossCutting.Database;
using RestApiTemplate.CrossCutting.DependencyInjection;

using Serilog;
using Serilog.Events;
using Serilog.Exceptions;
using Serilog.Formatting.Elasticsearch;
using Serilog.Sinks.Elasticsearch;

using Steeltoe.Discovery.Client;
using Steeltoe.Management.CloudFoundry;
using Steeltoe.Management.Endpoint.Env;
using Steeltoe.Management.Endpoint.Health;
using Steeltoe.Management.Endpoint.Info;
using Steeltoe.Management.Endpoint.Loggers;
using Steeltoe.Management.Endpoint.Mappings;
using Steeltoe.Management.Endpoint.Metrics;
using Steeltoe.Management.Endpoint.Refresh;
using Steeltoe.Management.Endpoint.Trace;

using Swashbuckle.AspNetCore.Swagger;

[assembly: ApiController]
namespace RestApiTemplate.Api
{
    /// <summary>
    /// Classe de inicialização do projeto
    /// <remarks>
    /// ExcludeFromCodeCoverage -  Utilize esta notation para excluir a classe do CodeCoverage
    /// [assembly: ApiController] - Responsável por validar os Dtos automaticamente, conforme configurado no Vallidation da classe
    /// https://docs.microsoft.com/pt-br/aspnet/core/web-api/?view=aspnetcore-2.2#apicontroller-attribute
    /// </remarks>
    /// </summary>
    [ExcludeFromCodeCoverage]
    public class Startup
    {
        private readonly IConfiguration _configuration;

#if RELEASE
        private readonly string PolicyNameForAllowedCors = "_myAllowSpecificOrigins";;
#endif

        public Startup(IConfiguration configuration)
        {
            _configuration = configuration;

            if (_configuration is null) return;

            Log.Logger = new LoggerConfiguration()
                    .MinimumLevel.Information()
#if DEBUG
                    .MinimumLevel.Debug()
#endif
                   .Destructure.UsingAttributes()
                   .Enrich.FromLogContext()
                   .Enrich.WithExceptionDetails()
                   .WriteTo.Elasticsearch(new ElasticsearchSinkOptions(new Uri(_configuration["ElasticConfiguration:Uri"]))
                   {
                       AutoRegisterTemplate = true,
                       CustomFormatter = new ElasticsearchJsonFormatter(),
                       ModifyConnectionSettings = settings => settings
                      .BasicAuthentication(_configuration["ElasticConfiguration:UserName"], _configuration["ElasticConfiguration:Password"])
                      .DisableAutomaticProxyDetection()
                   })
#if DEBUG
                   .WriteTo.File(_configuration["PathLog"], restrictedToMinimumLevel: LogEventLevel.Debug, outputTemplate: "{Timestamp:HH:mm} [{Level}] ({ThreadId}) {Message}  | CorrelationId:{CorrelationId}{NewLine}{Exception}")
#endif
                   .CreateLogger();
        }

        /// <summary>
        /// Configurações iniciais da aplicação
        /// </summary>
        /// <remarks>
        /// Foram cridas e utilizadas Extensions para facilitar as configurações padrões
        /// O projeto das Extensions é Rchlo.Omni.Common
        /// </remarks>
        /// <param name="services"></param>
        public void ConfigureServices(IServiceCollection services)
        {
#if RELEASE
            services.AddCorsConfiguration(PolicyNameForAllowedCors, "Origin1", "Origin2");
#endif

            services.AddMvcConfiguration()
                    .AddFluentValidation();

            services.AddApiVersioning();

            services.AddUserLoggedConfiguration();
            services.AddAppSettingsConfiguration(_configuration);
            services.AddAutomapperConfiguration();
            services.AddDependencyInjectionConfiguration();

            services.AddEFPostgresConfiguration(_configuration["ConnectionStrings:RchloConnection"]);

            services.AddDapperPostgresConfiguration(_configuration["ConnectionStrings:RchloConnection"]);

            services.AddSwaggerConfiguration(new Info
            {
                Version = _configuration["ApiDescriptions:Version"],
                Title = _configuration["ApiDescriptions:Title"],
                Description = _configuration["ApiDescriptions:Description"]
            });

            services.AddKeyCloakConfiguration
            (
                authority: _configuration["Authentication:Authority"],
                validIssuer: _configuration["Authentication:ValidIssuer"],
                validateIssuer: _configuration["Authentication:ValidateIssuer"].ToBoolean(),
                requireHttpsMetadata: _configuration["Authentication:RequireHttpsMetadata"].ToBoolean()
            );

            services.AddDiscoveryClient(_configuration);
            services.AddCloudFoundryActuators(_configuration);
            services.AddHealthActuator(_configuration);
            services.AddInfoActuator(_configuration);
            services.AddLoggersActuator(_configuration);
            services.AddTraceActuator(_configuration);
            services.AddRefreshActuator(_configuration);
            services.AddEnvActuator(_configuration);
            services.AddMappingsActuator(_configuration);
            services.AddMetricsActuator(_configuration);
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IHostingEnvironment env)
        {
#if RELEASE
            app.UseCors(PolicyNameForAllowedCors);
#endif

            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            else
            {
                // The default HSTS value is 30 days. You may want to change 
                // this for production scenarios, see https://aka.ms/aspnetcore-hsts.
                app.UseHsts();
            }

            app.UseSerilogRequestLogging();

            app.UseSwagger("Web API V1");

            app.UseAuthentication();

            app.UseStaticFiles();
            app.UseHttpsRedirection();
            app.UseMvc();

            app.UseDiscoveryClient();
            app.UseHealthActuator();
            app.UseInfoActuator();
            app.UseLoggersActuator();
            app.UseTraceActuator();
            app.UseRefreshActuator();
            app.UseEnvActuator();
            app.UseMappingsActuator();
            app.UseMetricsActuator();
            app.UseCloudFoundryActuators();
        }
    }
}
