﻿using System;
using System.Linq;
using System.Threading.Tasks;

using Microsoft.EntityFrameworkCore;

using Rchlo.Omni.Common.Adapter;
using Rchlo.Omni.Common.Extensions;
using Rchlo.Omni.Common.Models;

using RestApiTemplate.Application.Dtos.Request;
using RestApiTemplate.Application.Dtos.Response;
using RestApiTemplate.Application.Dtos.Settings;
using RestApiTemplate.Application.Services.Interfaces;
using RestApiTemplate.Domain.Constants;
using RestApiTemplate.Domain.Entities;

namespace RestApiTemplate.Application.Services
{
    /// <summary>
    /// As camadas de services devem ser utilizadas para comunicação com o banco de dados através do 
    /// Unit of Work que fica encarregado do gerenciamento das conexões, esta camada deve ser responsável pelas validações
    /// e regras de negócio e validações antes da persistência dos dados.
    /// </summary>
    public class ExampleService : IExampleService
    {
        private readonly IUnitOfWork _unitOfWork;
        private readonly IAdapter _adapter;
        private readonly UserLogged _userLogged;

        public ExampleService(IUnitOfWork unitOfWork, IAdapter adapter, UserLogged userLogged)
        {
            _unitOfWork = unitOfWork;
            _adapter = adapter;
            _userLogged = userLogged;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="example"></param>
        /// <returns></returns>
        public async Task<ReturnMessage<IPagedList<ExampleResponse>>> GetEnabledExamplesAsync(ExampleGetEnabledRequest example)
        {
            // Recupera o repositório que será trabalhado
            var exampleRepository = _unitOfWork.GetRepository<Example>();

            // Realiza as querys 
            var examples = await exampleRepository.GetPagedListAsync
            (
                predicate: _ => _.IsEnabled && _.Price >= example.PriceMinimum && _.Price <= example.PriceMaximum,
                orderBy: _ => _.OrderByDescending(__ => __.Price),
                pageSize: example.PageSize,
                pageIndex: example.PageIndex
            )
            .ConfigureAwait(true);

            // Realiza conversão da entidade para o objeto de saída
            var examplesResponse = _adapter.ConvertTo<PagedList<Example>, PagedList<ExampleResponse>>((PagedList<Example>)examples);

            // Retona objeto ReturnMessage<IPagedList<ExampleResponse>>
            return new ReturnMessage<IPagedList<ExampleResponse>>(examplesResponse);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="exampleRequest"></param>
        /// <remarks>
        /// Não é necessário realizar validação do Dto pois isso é feito
        /// automaticamente pelo atributo [ApiController] na Startup</remarks>
        /// https://docs.microsoft.com/pt-br/aspnet/core/web-api/?view=aspnetcore-2.2#apicontroller-attribute
        /// </returns>
        public async Task<ReturnMessage<ExampleResponse>> InsertExampleAsync(ExampleRequest exampleRequest)
        {
            exampleRequest?.SetUserLogged(_userLogged);

            // Converte o Dto de entrada para Entidade 
            var example = _adapter.ConvertTo<ExampleRequest, Example>(exampleRequest);

            // Recupera o repositório que será trabalhado
            var exampleRepository = _unitOfWork.GetRepository<Example>();

            // Realiza operação de inserção
            exampleRepository.Insert(example);

            // Realiza commit na transação
            await _unitOfWork.SaveChangesAsync().ConfigureAwait(false);

            // Converte o objeto da Entidade para o Dto de saída
            var exampleResponse = _adapter.ConvertTo<Example, ExampleResponse>(example);

            // Retona objeto ReturnMessage<ExampleResponse>
            return new ReturnMessage<ExampleResponse>(exampleResponse);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="exampleId"></param>
        /// <returns></returns>
        public async Task<ReturnMessage<ExampleResponse>> EnableExampleAsync(Guid exampleId)
        {
            // Recupera o repositório que será trabalhado
            var exampleRepository = _unitOfWork.GetRepository<Example>();

            // Realiza uma busca na base pelo Id informado
            var example = await exampleRepository.FindAsync(exampleId).ConfigureAwait(false);

            // Validação de objeto encontrado
            if (example is null)
                return new ReturnMessage<ExampleResponse>(ErrorsConstants.EXAMPLE_NAO_ENCONTRADO);
                    
            // Altera o valor do atributo
            example.Enable(_userLogged.Id?.ToGuid() ?? Guid.NewGuid());

            // Realiza operação de alteração
            exampleRepository.Update(example);

            // Realiza commit na transação
            await _unitOfWork.SaveChangesAsync().ConfigureAwait(false);

            // Converte o objeto da Entidade para o Dto de saída
            var exampleResponse = _adapter.ConvertTo<Example, ExampleResponse>(example);

            // Retona objeto ReturnMessage<ExampleResponse>
            return new ReturnMessage<ExampleResponse>(exampleResponse);
        }
    }
}