﻿#region Using
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;

using Rchlo.Omni.Common.Adapter;
using Rchlo.Omni.Common.Models;
using Rchlo.Omni.Common.Tests.Extensions;
using RestApiTemplate.Application.Dtos.Request;
using RestApiTemplate.Application.Services;
using RestApiTemplate.Domain.Constants;
using RestApiTemplate.Domain.Entities;

using Xunit;
using FakeItEasy;
#endregion

namespace RestApiTemplate.Tests.Application.Services
{
    /// <summary>
    /// Classe responsável por realizar os testes unitários do Application-Services
    /// </summary>
    /// <remarks>
    ///     Para nos auxiliar nas implementações dos testes, utilizamos 
    ///     
    ///     FakeItEasy - https://fakeiteasy.readthedocs.io/en/stable/
    /// </remarks>
    public class ExampleServiceTest : BaseTest
    {
        #region Variables
        private readonly ExampleService _exampleService;
        private readonly IUnitOfWork _unitOfWork;
        private readonly IRepository<Example> _exampleRepository;
        private readonly IAdapter _adapter;
        private readonly UserLogged _userLogged;
        #endregion

        #region Constructor
        public ExampleServiceTest()
        {
            _unitOfWork = A.Fake<IUnitOfWork>();
            _exampleRepository = A.Fake<IRepository<Example>>();
            _adapter = ServiceProvider.GetRequiredService<IAdapter>();
            _userLogged = A.Fake<UserLogged>(_ => _.WithArgumentsForConstructor
            (
                new object[]
                {
                    Guid.NewGuid().ToString(),
                    "User",
                    "user@email",
                    true
                }
            ));

            _exampleService = new ExampleService(_unitOfWork, _adapter, _userLogged);

            A.CallTo(() => _unitOfWork.GetRepository<Example>(false)).Returns(_exampleRepository);
        }
        #endregion

        #region Tests Methods
        [Fact]
        public void GetEnabledExamplesAsync_WithExistingFilters_ReturnsRecords()
        {
            //Arrange
            var exampleGetEnableRequest = GenerateExampleGetEnabledRequest();
            List<Example> examplesResult = new List<Example>() { GenerateExample() };
            var examplesPaged = examplesResult.ToPagedList(exampleGetEnableRequest.PageIndex, exampleGetEnableRequest.PageSize);

            A.CallTo(_exampleRepository.GetPagedListAsyncFunc<Example>()).Returns(examplesPaged);

            // Act
            var result = _exampleService.GetEnabledExamplesAsync(exampleGetEnableRequest).Result;

            // Assert
            A.CallTo(() => _unitOfWork.GetRepository<Example>(A<bool>.Ignored)).MustHaveHappenedOnceExactly();
            A.CallTo(_exampleRepository.GetPagedListAsyncAction<Example>()).MustHaveHappenedOnceExactly();
            
            Assert.NotNull(result.Data);
            Assert.True(result.Success);
        }

        [Fact]
        public async Task InsertExampleAsync_UsingCorrectsParameters_ReturnsOk()
        {
            //Arrange
            var exampleRequest = GenerateExampleRequest();

            // Act
            var result = await _exampleService.InsertExampleAsync(exampleRequest);

            // Assert
            A.CallTo(() => _exampleRepository.Insert(A<Example>.Ignored)).MustHaveHappenedOnceExactly();
            A.CallTo(() => _unitOfWork.SaveChangesAsync(false)).MustHaveHappenedOnceExactly();

            Assert.NotNull(result.Data);
            Assert.True(result.Success);
            Assert.False(result.Data.IsEnabled);
        }

        [Fact]
        public void EnableExampleAsync_ExistentItem_ReturnsOk()
        {
            // Arrange
            var example = GenerateExample();
            A.CallTo(() => _exampleRepository.FindAsync(example.Id)).Returns(example);

            // Act
            var result = _exampleService.EnableExampleAsync(example.Id).Result;

            // Assert
            A.CallTo(() => _exampleRepository.Update(A<Example>.That.Matches(_ => _.Id.Equals(example.Id)))).MustHaveHappenedOnceExactly();
            A.CallTo(() => _unitOfWork.SaveChangesAsync(false)).MustHaveHappenedOnceExactly();

            Assert.NotNull(result.Data);
            Assert.True(result.Success);
            Assert.Equal(result.Data.Id, example.Id);
            Assert.True(result.Data.IsEnabled);
        }

        [Fact]
        public void EnableExampleAsync_InexistentItem_ReturnsItemNotFoundMessage()
        {
            // Arrange
            Example example = null;
            A.CallTo(() => _exampleRepository.FindAsync(Guid.Empty)).Returns(example);

            // Act
            var result = _exampleService.EnableExampleAsync(Guid.Empty).Result;

            // Assert
            A.CallTo(() => _unitOfWork.SaveChangesAsync(false)).MustNotHaveHappened();
            Assert.Null(result.Data);
            Assert.Contains(ErrorsConstants.EXAMPLE_NAO_ENCONTRADO, result.ErrorMessages);
            Assert.False(result.Success);
        }
        #endregion

        #region Private Methods

        private ExampleRequest GenerateExampleRequest(string name = "Name Example", decimal price = 1)
        {
            return new ExampleRequest()
            {
                Name = name,
                Price = price
            };
        }

        private Example GenerateExample()
        {
            var exampleRequestResquest = GenerateExampleRequest();
            return _adapter.ConvertTo<ExampleRequest, Example>(exampleRequestResquest);
        }

        private ExampleGetEnabledRequest GenerateExampleGetEnabledRequest()
        {
            return new ExampleGetEnabledRequest()
            {
                PageIndex = 0,
                PageSize = 1,
                PriceMaximum = 2,
                PriceMinimum = 1
            };
        }
        #endregion
    }
}
