﻿using FluentValidation;
using FluentValidation.TestHelper;
using RestApiTemplate.Application.Dtos.Request;
using RestApiTemplate.Application.Validations;
using RestApiTemplate.Domain.Constants;
using Xunit;

namespace RestApiTemplate.Tests.Application.Validations
{
    /// <summary>
    /// Classe responsável por testar as Validation dos Dto´s
    /// </summary>
    /// <remarks>
    ///     Como os Dto´s são validados de forma automática ([assembly: ApiController] - Startup), 
    ///     temos a necessidade de ter uma classe de testes exclusivas para os Validations
    ///     
    ///     Utilizamos o FluentValidation.TestHelper para auxiliar nos testes
    ///         Link de referência FluentValidation.TestHelper: https://fluentvalidation.net/testing
    /// </remarks>
    public class ExampleGetEnableRequestValidationTest
    {
        #region Variables
        private readonly IValidator<ExampleGetEnabledRequest> _exampleGetEnabledRequest;
        #endregion

        #region Constructor
        public ExampleGetEnableRequestValidationTest()
        {
            _exampleGetEnabledRequest = new ExampleGetEnableRequestValidation();
        }
        #endregion

        #region Test Methods
        [Theory]
        [InlineData(0)]
        [InlineData(11)]
        public void ExampleRequestValidation_InvalidPriceMinimunParameter_ReturnsInvalidPriceMessage(decimal priceMinimum)
        {
            _exampleGetEnabledRequest.ShouldHaveValidationErrorFor(_ => _.PriceMinimum, priceMinimum);
        }

        [Theory]
        [InlineData(1)]
        [InlineData(2)]
        public void ExampleRequestValidation_PriceMinimunParameter_ReturnsValidPriceMessage(decimal priceMinimum)
        {
            _exampleGetEnabledRequest.ShouldNotHaveValidationErrorFor(_ => _.PriceMinimum, priceMinimum);
        }

        [Theory]
        [InlineData(0)]
        [InlineData(11)]
        public void ExampleRequestValidation_ValidPriceMaximumParameter_ReturnsInvalidPriceMessage(decimal priceMaximum)
        {
            _exampleGetEnabledRequest.ShouldHaveValidationErrorFor(_ => _.PriceMaximum, priceMaximum);
        }

        [Theory]
        [InlineData(1)]
        [InlineData(2)]
        public void ExampleRequestValidation_ValidPriceMaximumParameter_ReturnsValidPriceMessage(decimal priceMaximum)
        {
            _exampleGetEnabledRequest.ShouldNotHaveValidationErrorFor(_ => _.PriceMaximum, priceMaximum);
        }

        [Fact]
        public void ExampleRequestValidation_InvalidPageIndex_ReturnsNOk()
        {
            _exampleGetEnabledRequest.ShouldHaveValidationErrorFor(_ => _.PageIndex, -1);
        }

        [Fact]
        public void ExampleRequestValidation_ValidPageIndex_ReturnsOk()
        {
            _exampleGetEnabledRequest.ShouldNotHaveValidationErrorFor(_ => _.PageIndex, BussinessConstants.MIN_VALUE_PAGE_INDEX);
        }

        [Fact]
        public void ExampleRequestValidation_PriceMinimumGreaterThanPriceMaximum_ReturnsInvalidMessage()
        {
            //Arrange 
            ExampleGetEnabledRequest _exampleGetEnabled = GenerateExampleGetEnabledRequest_PriceMinimumGreaterThanPriceMaximum();

            // Act
            var act = _exampleGetEnabledRequest.Validate(_exampleGetEnabled);

            // Assert
            Assert.False(act.IsValid);
            act.Errors.WithErrorMessage(ErrorsConstants.MIN_VALULE_GREATER_THAN_MAX_VALUE);
            
        }
        #endregion

        #region Private Methods
        private ExampleGetEnabledRequest GenerateExampleGetEnabledRequest_PriceMinimumGreaterThanPriceMaximum()
        {
            return new ExampleGetEnabledRequest()
            {
                PageIndex = 0,
                PageSize = 10,
                PriceMaximum = 1,
                PriceMinimum = 2
            };
        }
        #endregion
    }
}
